/*
 * main.c
 *
 * Created: 4/13/2013 12:26:00 PM
 *  Author: don
 */

#include "main.h"
#include "ps2_kbd.h"
#include "Keyboard.h"

// currently-pressed keys and modifiers (unused keys are set to 0x00)
uint8_t keys[6];	// USB HID protocol only supports 6 simultaneously-pressed keys
uint8_t modifiers = 0;

int main(void)
{
	uint8_t i;
	uint8_t hid_scancode;	// scancode of pressed or released key
	bool pressed;			// true if key was pressed, false if released
	bool modifier;			// true if key was a modifier

	// clear pressed-key list
	for (i = 0; i < 6; i++) keys[i] = 0;

	// setup LED outputs
	LED_PORT_DIR |= STATUS_LED | ACTION_LED;
	LED_PORT &= ~(STATUS_LED | ACTION_LED);		// both LEDs off until USB initialized
	kbd_init ();
	USB_Init ();
	sei ();

	// main loop
	while (true) {
		KeyboardTasks ();
		if (kbd_poll (&hid_scancode, &modifier, &pressed)) {
			// if key pressed, add or remove it from the list
			if (modifier) {
				// modifiers are stored as bitmask
				if (pressed) modifiers |= hid_scancode;
				else modifiers &= ~hid_scancode;
			} else {
				// keys are stored in list
				if (pressed) {
					// search for an empty spot to store it
					for (i = 0; i < 6; i++) {
						if (keys[i] == 0x00) {
							keys[i] = hid_scancode;
							break;
						}
					}
					// if it can't fit it gets ignored
				} else {
					// search for where it currently is, to remove it
					for (i = 0; i < 6; i++) {
						if (keys[i] == hid_scancode) {
							keys[i] = 0x00;
						}
					}
					// if it isn't found it gets ignored
				}
			}
		}
	}
}

void EVENT_USB_Device_Connect(void)
{
//	LED_PORT |= STATUS_LED;
}

void EVENT_USB_Device_Disconnect(void)
{
//	LED_PORT &= ~STATUS_LED;
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= HID_Device_ConfigureEndpoints(&Keyboard_HID_Interface);

	USB_Device_EnableSOFEvents();

	//if (! ConfigSuccess) LED_PORT &= ~STATUS_LED;
}


/** HID class driver callback function for the creation of HID reports to the host.
 *
 *  \param[in]     HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in,out] ReportID    Report ID requested by the host if non-zero, otherwise callback should set to the generated report ID
 *  \param[in]     ReportType  Type of the report to create, either HID_REPORT_ITEM_In or HID_REPORT_ITEM_Feature
 *  \param[out]    ReportData  Pointer to a buffer where the created report should be stored
 *  \param[out]    ReportSize  Number of bytes written in the report (or zero if no report is to be sent
 *
 *  \return Boolean true to force the sending of the report, false to let the library determine if it needs to be sent
 */
bool CALLBACK_HID_Device_CreateHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo, uint8_t* const ReportID,
                                         const uint8_t ReportType, void* ReportData, uint16_t* const ReportSize)
{
	uint8_t i;
	USB_KeyboardReport_Data_t* KeyboardReport = (USB_KeyboardReport_Data_t*)ReportData;

	uint8_t UsedKeyCodes = 0;

	KeyboardReport->Modifier = modifiers;

	for (i = 0; i < 6; i++) {
		if (keys[i]) KeyboardReport->KeyCode[UsedKeyCodes++] = keys[i];
	}

	*ReportSize = sizeof(USB_KeyboardReport_Data_t);
	return false;
}

/** HID class driver callback function for the processing of HID reports from the host.
 *
 *  \param[in] HIDInterfaceInfo  Pointer to the HID class interface configuration structure being referenced
 *  \param[in] ReportID    Report ID of the received report from the host
 *  \param[in] ReportType  The type of report that the host has sent, either HID_REPORT_ITEM_Out or HID_REPORT_ITEM_Feature
 *  \param[in] ReportData  Pointer to a buffer where the created report has been stored
 *  \param[in] ReportSize  Size in bytes of the received HID report
 */
void CALLBACK_HID_Device_ProcessHIDReport(USB_ClassInfo_HID_Device_t* const HIDInterfaceInfo,
                                          const uint8_t ReportID,
                                          const uint8_t ReportType,
                                          const void* ReportData,
                                          const uint16_t ReportSize)
{
//	uint8_t send = 0;
	uint8_t* LEDReport = (uint8_t*)ReportData;
/*	if (*LEDReport & HID_KEYBOARD_LED_CAPSLOCK) send |= 0x04;
	if (*LEDReport & HID_KEYBOARD_LED_NUMLOCK) send |= 0x02;
	if (*LEDReport & HID_KEYBOARD_LED_SCROLLLOCK) send |= 0x01;

	kbd_set_leds (send);*/
	if (*LEDReport & HID_KEYBOARD_LED_CAPSLOCK) LED_PORT |= STATUS_LED;
	else LED_PORT &= ~STATUS_LED;

	if (*LEDReport & HID_KEYBOARD_LED_NUMLOCK) LED_PORT |= ACTION_LED;
	else LED_PORT &= ~ACTION_LED;
}
